==> ducene-0.0.1.tar.gz: lib/src/index_searcher.dart <==
// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'index_reader.dart';
import 'query.dart';
import 'document.dart';

class IndexSearcher {

  final IndexReader _reader;

  IndexSearcher(this._reader);

  int count(BoolQuery query) => search(query, _reader.maxDoc(), ScoreSort.indexed()).totalHits;
  Document doc(int docID, Iterable<String> fieldsToLoad) => _reader.document(docID, fieldsToLoad);

  TopDocs search(BoolQuery query, int n, ScoreSort scoreSort) {
    Set<int> filteredDocSet = new Set();
    int i = 0;
    for (TermQuery termQuery in query.filters) {
      var docIds = _reader.getTerms(termQuery.term.field)[termQuery.term.text];
      if (docIds == null) docIds = [];
      Set target = new Set()..addAll(docIds);
      if (i == 0) {
        filteredDocSet = target;
        i++;
        continue;
      }
      Set<int> unionSet = new Set.from(filteredDocSet);
      filteredDocSet = unionSet.intersection(target);
      i++;
    }
    if (i > 0 && filteredDocSet.isEmpty) {
      return new TopDocs(0, [new ScoreDoc(-1, double.NAN)], filteredDocSet);
    }

    Set<int> curDocSet = new Set();
    bool isFirst = true;
    for (TermQuery termQuery in query.termQueries) {
      var docIds = _reader.getTerms(termQuery.term.field)[termQuery.term.text];
      if (docIds == null) docIds = [];
      Set target = new Set()..addAll(docIds);
      if (isFirst) {
        curDocSet = target;
        isFirst = false;
        continue;
      }
      Set<int> unionSet = new Set.from(curDocSet);
      switch (query.op) {
        case Op.and:
          curDocSet = unionSet.intersection(target);
          break;
        case Op.or:
          curDocSet = unionSet.union(target);
          break;
      }
    }

    if (!filteredDocSet.isEmpty) {
      Set<int> unionSet = new Set.from(filteredDocSet);
      curDocSet = unionSet.intersection(curDocSet);
    }

    Map<int,double> docIdScore = scoreSort.scoreThenSort(curDocSet, _reader);

    int totalHits = docIdScore.length;
    int max = (totalHits < n) ? totalHits : n;
    List<ScoreDoc> scoreDocs = [];
    for (int docId in docIdScore.keys.toList().getRange(0, max)) {
      ScoreDoc scoreDoc = new ScoreDoc(docId, docIdScore[docId]);
      scoreDocs.add(scoreDoc);
    }
    return new TopDocs(totalHits, scoreDocs, curDocSet);
  }

}

class TopDocs {
  final int totalHits;
  final List<ScoreDoc> scoreDocs;
  final Set<int> docSet;
  TopDocs(this.totalHits, this.scoreDocs, this.docSet);
}

class ScoreDoc {
  final int doc;
  final double score;
  ScoreDoc(this.doc, this.score);
}

abstract class ScoreSort {
  static indexed() => new DocIdSort(false);
  static reversed() => new DocIdSort(true);
  Map<int,double> scoreThenSort(Set<int> docSet, IndexReader reader);
}

class DocIdSort extends ScoreSort {
  final bool _reversed;
  DocIdSort(this._reversed);
  Map<int,double> scoreThenSort(Set<int> docSet, IndexReader reader) {
    var docList = docSet.toList();
    if (!_reversed) {
      docList.sort();
      return new Map.fromIterable(docList, key: (e) => e, value: (e) => 1.00);
    } else {
      return new Map.fromIterable(docList.reversed.toList(), key: (e) => e, value: (e) => 1.00);
    }
  }
}

class MatchScoreSort extends ScoreSort {
  final BoolQuery _bq;
  MatchScoreSort(this._bq);
  Map<int,double> scoreThenSort(Set<int> docSet, IndexReader reader) {
    Map<int,double> m = {};
    for (var tq in _bq.termQueries) {
      var terms = reader.getTerms(tq.term.field);
      if (terms == null) continue;
      List<int> docIds = terms[tq.term.text];
      if (docIds == null) continue;
      for (int docId in docIds) {
        if (docSet.contains(docId)) {
          if (m[docId] == null) {
            m[docId] = (1.00 * tq.term.boost) + (docId * 0.00000001);
          } else {
            m[docId] = m[docId] + (1.00 * tq.term.boost);
          }
        }
      }
    }
    List<Map<int,double>> l = [];
    for (var k in m.keys) {
      Map<int,double> x = {};
      x[k] = m[k];
      l.add(x);
    }
    l.sort((a, b) => b.values.toList()[0] - a.values.toList()[0]);
    return new Map.fromIterable(l, key: (e) => (e as Map).keys.toList()[0], value: (e) => (e as Map).values.toList()[0]);
  }
}

==> ducene-0.0.2.tar.gz: lib/src/index_searcher.dart <==
// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of ducene;

class IndexSearcher {

  final IndexReader _reader;

  IndexSearcher(this._reader);

  int count(BoolQuery query) => search(query, _reader.maxDoc(), ScoreSort.indexed()).totalHits;
  Document doc(int docID, Iterable<String> fieldsToLoad) => _reader.document(docID, fieldsToLoad);

  TopDocs search(BoolQuery query, int n, ScoreSort scoreSort) {
    Set<int> filteredDocSet = new Set();
    int i = 0;
    for (TermQuery termQuery in query.filters) {
      var docIds = _reader.getTerms(termQuery.term.field)[termQuery.term.text];
      if (docIds == null) docIds = [];
      Set target = new Set()..addAll(docIds);
      if (i == 0) {
        filteredDocSet = target;
        i++;
        continue;
      }
      Set<int> unionSet = new Set.from(filteredDocSet);
      filteredDocSet = unionSet.intersection(target);
      i++;
    }
    if (i > 0 && filteredDocSet.isEmpty) {
      return new TopDocs(0, [new ScoreDoc(-1, double.NAN)], filteredDocSet);
    }

    Set<int> curDocSet = new Set();
    bool isFirst = true;
    for (TermQuery termQuery in query.termQueries) {
      var docIds = _reader.getTerms(termQuery.term.field)[termQuery.term.text];
      if (docIds == null) docIds = [];
      Set target = new Set()..addAll(docIds);
      if (isFirst) {
        curDocSet = target;
        isFirst = false;
        continue;
      }
      Set<int> unionSet = new Set.from(curDocSet);
      switch (query.op) {
        case Op.and:
          curDocSet = unionSet.intersection(target);
          break;
        case Op.or:
          curDocSet = unionSet.union(target);
          break;
      }
    }

    if (!filteredDocSet.isEmpty) {
      Set<int> unionSet = new Set.from(filteredDocSet);
      curDocSet = unionSet.intersection(curDocSet);
    }

    Map<int,double> docIdScore = scoreSort.scoreThenSort(curDocSet, _reader);

    int totalHits = docIdScore.length;
    int max = (totalHits < n) ? totalHits : n;
    List<ScoreDoc> scoreDocs = [];
    for (int docId in docIdScore.keys.toList().getRange(0, max)) {
      ScoreDoc scoreDoc = new ScoreDoc(docId, docIdScore[docId]);
      scoreDocs.add(scoreDoc);
    }
    return new TopDocs(totalHits, scoreDocs, curDocSet);
  }

}

class TopDocs {
  final int totalHits;
  final List<ScoreDoc> scoreDocs;
  final Set<int> docSet;
  TopDocs(this.totalHits, this.scoreDocs, this.docSet);
}

class ScoreDoc {
  final int doc;
  final double score;
  ScoreDoc(this.doc, this.score);
}

abstract class ScoreSort {
  static ScoreSort indexed() => new DocIdSort(false);
  static ScoreSort reversed() => new DocIdSort(true);
  Map<int,double> scoreThenSort(Set<int> docSet, IndexReader reader);
}

class DocIdSort implements ScoreSort {
  final bool _reversed;
  DocIdSort(this._reversed) : super();
  Map<int,double> scoreThenSort(Set<int> docSet, IndexReader reader) {
    var docList = docSet.toList();
    if (!_reversed) {
      docList.sort();
      return new Map.fromIterable(docList, key: (e) => e, value: (e) => 1.00);
    } else {
      return new Map.fromIterable(docList.reversed.toList(), key: (e) => e, value: (e) => 1.00);
    }
  }
}

class MatchScoreSort implements ScoreSort {
  final BoolQuery _bq;
  MatchScoreSort(this._bq) : super();
  Map<int,double> scoreThenSort(Set<int> docSet, IndexReader reader) {
    Map<int,double> m = {};
    for (var tq in _bq.termQueries) {
      var terms = reader.getTerms(tq.term.field);
      if (terms == null) continue;
      List<int> docIds = terms[tq.term.text];
      if (docIds == null) continue;
      for (int docId in docIds) {
        if (docSet.contains(docId)) {
          if (m[docId] == null) {
            m[docId] = (1.00 * tq.term.boost) + (docId * 0.00000001);
          } else {
            m[docId] = m[docId] + (1.00 * tq.term.boost);
          }
        }
      }
    }
    List<Map<int,double>> l = [];
    for (var k in m.keys) {
      Map<int,double> x = {};
      x[k] = m[k];
      l.add(x);
    }
    l.sort((a, b) => b.values.toList()[0] - a.values.toList()[0]);
    return new Map.fromIterable(l, key: (e) => (e as Map).keys.toList()[0], value: (e) => (e as Map).values.toList()[0]);
  }
}
